﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Operators {
  [Item("SubScopesRandomCreator", "Creates pseudo random number generators for each subscope.")]
  [StorableClass]
  public sealed class SubScopesRandomCreator : SingleSuccessorOperator {
    #region Parameter Properties
    private LookupParameter<IRandom> GlobalRandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["GlobalRandom"]; }
    }
    private ScopeParameter CurrentScopeParameter {
      get { return (ScopeParameter)Parameters["CurrentScope"]; }
    }
    private IScope CurrentScope {
      get { return CurrentScopeParameter.ActualValue; }
    }
    #endregion

    [StorableConstructor]
    private SubScopesRandomCreator(bool deserializing) : base(deserializing) { }
    private SubScopesRandomCreator(SubScopesRandomCreator original, Cloner cloner)
      : base(original, cloner) {
    }
    public SubScopesRandomCreator()
      : base() {
      #region Create parameters
      Parameters.Add(new ScopeParameter("CurrentScope", "The current scope into which the random number generator is cloned."));
      Parameters.Add(new LookupParameter<IRandom>("GlobalRandom", "A pseudo random number generator.", "Random"));
      #endregion
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SubScopesRandomCreator(this, cloner);
    }

    public override IOperation Apply() {
      IRandom random = GlobalRandomParameter.ActualValue;
      IRandom subScopesRandom = (IRandom)random.Clone();
      subScopesRandom.Reset(random.Next());
      CurrentScope.Variables.Add(new Variable("Random", "A pseudo random number generator.", subScopesRandom));
      return base.Apply();
    }
  }
}
